module CyberCoach
  #
  # Mixin to make a resource readable in pages.
  # Include it in a class by to use it.
  #
  module Pageable
    #
    # Installs class and instance methods in the class it is included in.
    #
    def self.included(base)
      base.extend ClassMethods
      base.send :include, InstanceMethods
    end

    #
    # The class methods to install.
    #
    module ClassMethods
      #
      # :category: CRUD
      #
      # Returns the first page of resources.
      # options:: A hash of options to send with the request.
      # invalidate:: Invalidates it when true, skips invalidation when false.
      #
      def read_all(options = {}, invalidate = true)
        page = ResourcePage.new(type: self)
        page.read(options, invalidate)
        page
      end
    end

    #
    # The instance methods to install.
    #
    module InstanceMethods
    end
  end
end
