module CyberCoach
  #
  # Mixin for a resource that gets its URI assigned from the client.
  # Include it in a class to use it.
  #
  module PutCreateable
    #
    # Installs class and instance methods in the class it is included in.
    #
    def self.included(base)
      base.extend ClassMethods
      base.send :include, InstanceMethods
    end

    #
    # The class methods to install.
    #
    module ClassMethods
    end

    #
    # The instance methods to install.
    #
    module InstanceMethods
      #
      # :category: CRUD
      #
      # Creates it.
      # Reads itself from the response.
      # Raises HttpError if the request is unsuccessful.
      # options:: A hash of options to send with the request.
      # invalidate:: Invalidates it when true, skips invalidation when false.
      #
      def create(options = {}, invalidate = true)
        if invalidate
          self.invalidate
        end
        options = @options.merge(options).merge(
          body: serialize
        )
        response = self.class.put(@uri, options)
        if response.success?
          deserialize(response)
        else
          fail HttpError, response.response
        end
      end
    end
  end
end
