module CyberCoach
  #
  # A Sport has Subscriptions to which Entries are submitted.
  #
  class Sport < Resource
    #
    # It is pageable.
    #
    include Pageable

    #
    # :attr: name
    # The name.
    #

    #
    # :attr: description
    # The description.
    #

    #
    # :attr: subscriptions
    # The Subscriptions to it.
    #

    attr_accessor :name, :description, :subscriptions

    #
    # :category: Serialization
    #
    # Creates itself from a serializable representation, which only contains
    # simple data types.
    # serializable:: A hash with the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * name:: The name.
    # * description:: The description.
    # * subscriptions:: Subscription serializables.
    #
    def from_serializable(serializable)
      super(serializable)
      @name = serializable['name']
      @description = serializable['description']
      @subscriptions = []
      unless serializable['subscriptions'].nil?
        @subscriptions = serializable['subscriptions'].map do
        |subscription_serializable|
          subscription = Subscription.new
          subscription.from_serializable(subscription_serializable)
          subscription
        end
      end
    end

    #
    # :category: Serialization
    #
    # Returns a serializable representation, which only contains simple data
    # types.
    # The hash has the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * name:: The name.
    # * description:: The description.
    #
    def to_serializable
      serializable = super
      serializable['name'] = @name
      serializable['description'] = @description
      serializable
    end

    #
    # :category: Configuration
    #
    # Returns 'sport'.
    #
    def singular_name
      'sport'
    end

    #
    # :category: Configuration
    #
    # Returns 'sports'.
    #
    def plural_name
      'sports'
    end

    protected

    #
    # :category: Invalidation
    #
    # Sets the uri to the base uri and the name.
    #
    def invalidate_uri
      unless @name.nil?
        @uri = "#{resource_base_uri}#{@name}/"
      end
    end

    def initializable_with
      super + [:name, :description]
    end
  end
end
